/*

  Programma Eq_iperbolica.ccp:
  Programma che integra una equazione parabolica
  con uno schema di Runge-Kutta al secondo ordine.

  d f / dt - c * d f / dx = 0

  in un intervallo [0,2*pi] con condizioni al bordo
  di tipo periodico.

  Il programma legge i dati di input dal file: input.data.

 */

#include <iostream>
#include <fstream>
#include <cmath>
#include "VarX.h"
using namespace std;


/*

  Programma principale

*/

int main()
{

  // Lettura dati iniziali dal file
  ifstream finp;
  finp.open( "input.data" );

  // Lettura numero di punti griglia
  long int numPoints;
  finp >> numPoints;

  // Lettura passo temporale della simulazione
  double dt;
  finp >> dt;

  // Lettura durata totale della simulazione
  double tEnd;
  finp >> tEnd;

  // Lettura intervallo tra le stampe in output
  double tPrint;
  finp >> tPrint;

  // Lettura parametro c
  double c;
  finp >> c;

  // Lettura parametro k
  long int k;
  finp >> k;

  // Chiusura file di input
  finp.close();


  // Variabili per griglia spaziale
  double pi = acos( -1.0 );
  double a, b;
  a = 0.0;
  b = 2.0 * pi;


  // Variabili per ciclo temporale
  long nEnd   = long( tEnd / dt );
  long nPrint = long( tPrint / dt );


  // Costruzione dei vettori di lavoro
  double *x  = new double[numPoints];
  double *f  = new double[numPoints];
  double *fs = new double[numPoints];
  double *df = new double[numPoints];


  // Definizione dei punti griglia e della condizione iniziale
  double dx = ( b - a ) / ( numPoints - 1 );
  for ( long int i = 0; i < numPoints; i++ )
    {
      x[i] = a + i * dx;
      f[i] = sin( k * x[i] );
    }

  // Oggetto della classe VarX per calcolo delle derivate
  VarX ff = VarX( numPoints, f, 2, dx );

  double tCurr = 0.0, tNext;

  // Ciclo temporale

  for ( long int nt = 1; nt <= nEnd; nt++ )
    {

      tCurr = ( nt - 1 ) * dt;
      tNext = nt * dt;

      // Primo step del Runge-Kutta

      // Calcolo derivata seconda
      ff.getDerX( df );

      // Calcolo variabile al passo intermedio
      //for ( long int i = 0; i < numPoints; i++ )
      //fs[i] = f[i] + 0.5 * dt * c * df[i];

      // Valore del campo al passo intermedio
      //ff.setValue( fs );


      // Secondo step del Runge-Kutta

      // Calcolo della derivata seconda
      //ff.getDerX( df );

      // Calcolo campo al tempo successivo
      for ( long int i = 0; i < numPoints; i++ )
	f[i] = f[i] + dt * c * df[i];


      // Valore del campo al passo finale
      ff.setValue( f );


      // Stampa ogni Tprint intervalli di stampa

      if ( ( nt % nPrint ) == 0 )
	{
	  cout << "set title 't = " << tNext << "'" << endl;
	  cout << "set xlabel 'x'" << endl;
	  cout << "set ylabel 'f'" << endl;
	  cout << "set xrange [" << a << ":" << b << "]" << endl;
	  cout << "set yrange [-1:+1]" << endl;
	  cout << "plot '-' using 1:2 notitle with lines" << endl;
	  for ( long int i = 0; i < numPoints; i++ )
	    cout << x[i] << "   " << f[i] << endl;
	  cout << "e" << endl;
	}

    }

}
